%+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
% Create cells:
% P1: [0,1] * [0,1]
%   [ 1  0]       [1]
%   [ 0  1] x < = [1]
%   [-1  0]       [0]
%   [ 0 -1]       [0]
% P2: [1,2] * [0,1]
%   [ 1  0]       [ 2]
%   [ 0  1] x < = [ 1]
%   [-1  0]       [-1]
%   [ 0 -1]       [ 0]
%+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
P1 = polytope([eye(2); -eye(2)], [1; 1; 0; 0]);
P2 = polytope([eye(2); -eye(2)], [2; 1; -1; 0]);


%+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
% Set up the optimal control problem: 
%                          N-1
%                          ___
%       ||        ||       \   ||      ||      ||      ||
%  min  ||P_N x(N)||    +  /   ||Q x(k)||    + ||R u(k)||     
%   U   ||        || p     --- ||      || p    ||      || p
%                          k=0
% 
%  subject to:
%     x(k+1) = fdyn( x(k), u(k), w(k) )
%%    x(N) \in Tset
%+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

% Specify the dynamics
%  x(k+1)=Ax(k)+Bu(k)
%  y(k)=Cx(k)+Du(k)
sysStruct.A= [1.0000 0.0952; 0 0.9048];
sysStruct.B= [0.0048; 0.0952];
sysStruct.C= eye(2);
sysStruct.D= zeros(2,1);

% Set constraints on output
sysStruct.ymin    =   [0; -1];
sysStruct.ymax    =   [3; 1];

% Set constraints on input
sysStruct.umin    =   -sqrt(0.5);
sysStruct.umax    =   sqrt(0.5);

% String labels for states, inputs and outputs
sysStruct.StateName = { 's1', 's2' }; 
sysStruct.InputName = { 'u' };
sysStruct.OutputName = { 's1', 's2' };

% Specify the cost
%                           N-1
%                          ___
%            T             \        T              T
%   min  x(N)  P_N x(N) +  /    x(k)  Q x(k) + u(k)  R u(k)
%    U                     ---
%                          k=0

% Use 2-norm
probStruct.norm = 2;

% Set the prediction horizon
probStruct.N = 5;

% Set the weight matrices
probStruct.Q = zeros(size(sysStruct.A,2));
probStruct.R = eye(size(sysStruct.B,2));
probStruct.P_N = 10*eye(size(sysStruct.A,2));

% Set the terminal set
probStruct.Tset = P2;

% Set the reference point
probStruct.xref = [1.5; 0];

% Construct constraints and objective
[C, O, V] = mpt_ownmpc(sysStruct, probStruct);

% Compute the set of "safe" state using set union
Psafe = union([P1, P2]);

% Now add constraints telling that each predicted state must lie inside of the
% "safe" set of states
for k = 1:length(V.x),
    C = C + set(ismember(V.x{k}, Psafe));
end

% Compute an explicit controller with custom constraints
ctrl = mpt_ownmpc(sysStruct, probStruct, C, O, V);

% Obtain the feasible set of the controller
feasInitSet = ctrl.Pfinal;

plot(P1, 'w');
hold on;
plot(P2, 'w');
plot(feasInitSet, 'g');
plot([1 1], [0 1], 'k-');
axis([0 2 0 1]);


% Plot a trajectory
x0 = [0.9; 0.9];
[x_open, u_open] = sim(ctrl, x0, struct('openloop', 1));
plot(x_open(:,1), x_open(:,2), 'ko-');
hold off;